@@ -28,8 +28,9 @@ class OptionClass:
 
 class InstrumentDialog(QtGui.QDialog):
 
-	def __init__(self):
+	def __init__(self, item):
 		QtGui.QDialog.__init__(self)
+		self.item = item
 		self.ui = Ui_instrumentDialog()
 		self.ui.setupUi(self)
 		self.setup()
@@ -44,14 +45,53 @@ class InstrumentDialog(QtGui.QDialog):
 			self.ui.midi.addItem("%d. %s" % (d, x))
 			d += 1
 
+		self.connect(self.ui.algorithm, 
+			QtCore.SIGNAL("activated(int)"),
+			lambda x: self.load_instrument(self.ui.algorithm.currentText()))
+
+		self.connect(self.ui.letring, 
+			QtCore.SIGNAL("stateChanged(int)"),
+			lambda x: self.ui.noteduration.setEnabled(x))
+
+		self.connect(self.ui.buttonBox,
+			QtCore.SIGNAL("accepted()"),
+			lambda: self.save_instrument())
+
+	def save_instrument(self):
+		chan = self.ui.channel.value()
+		drum = False
+		if chan == 9:
+			drum = True
+
+
+		res = ""
+		res += self.ui.algorithm.currentText() + " { "
+		res += "channel:%d " % (self.ui.channel.value())
+		if not drum:
+			res += "midi_instr:%d " % (self.ui.midi.currentIndex())
+		res += "start:%d " % self.ui.stepstart.value()
+		res += "chance:%d " % (self.ui.chance.value())
+		if self.ui.step.value() != 0:
+			res += "step:%d " % self.ui.step.value()
+		if self.ui.stepend.value() != -1:
+			res += "end:%d " % self.ui.stepend.value()
+		res += "max_velocity:%d min_velocity:%d " % (self.ui.maxvelocity.value(),
+				self.ui.minvelocity.value())
+		if self.ui.letring.isChecked():
+			res += "let_ring:1 note_length:%d " % self.ui.noteduration.value()
+		res += "}"
+		self.item.setText(res)
+
 	def load_instrument(self, instr_str):
+		instr_str = str(instr_str)
 		parts = instr_str.split(" ")
 
 		index = self.combo_index_by_text(self.ui.algorithm, parts[0])
 		if index > 0:
 			self.ui.algorithm.setCurrentIndex(index)
 
-		i = getattr(Musicians, parts[0])({})
+		params = Options.parse_instrument_params(parts[1:])
+		i = getattr(Musicians, parts[0])(params)
 		self.ui.stepstart.setValue(i.start)
 		self.ui.stepend.setValue(i.end)
 		self.ui.step.setValue(i.step)
@@ -76,9 +116,9 @@ class InstrumentDialog(QtGui.QDialog):
 			self.ui.letring.setChecked(True)
 			self.ui.noteduration.setEnabled(True)
 			if 'note_length' in i.params:
-				self.ui.noteduration = i.params["note_length"]
+				self.ui.noteduration.setValue(i.params["note_length"])
 			else:
-				self.ui.noteduration = 2
+				self.ui.noteduration.setValue(2)
 		else:
 			self.ui.letring.setChecked(False)
 			self.ui.noteduration.setEnabled(False)
@@ -160,6 +200,9 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.removeinstrument,
 			QtCore.SIGNAL("clicked()"),
 			self.remove_instrument)
+		self.connect(self.ui.clearinstruments,
+			QtCore.SIGNAL("clicked()"),
+			self.ui.instruments.clear)
 		self.connect(self.ui.editinstrument,
 			QtCore.SIGNAL("clicked()"),
 			self.edit_instrument)
@@ -224,7 +267,7 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		cur = self.ui.instruments.item(self.ui.instruments.currentRow())
 		if cur is None:
 			return
-		i = InstrumentDialog()
+		i = InstrumentDialog(self.ui.instruments.currentItem())
 		i.load_instrument(str(cur.text()))
 		i.show()
 		i.exec_()
@@ -346,7 +389,8 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		if o.instrument != "":
 			instr = o.instrument.split(",")
 			for i in instr:
-				if i in Options.get_available_instruments():
+				parts =i.split(" ")
+				if parts[0] in Options.get_available_instruments():
 					self.ui.instruments.addItem(i)
 		if o.progression != "":
 			prog = o.progression.split(",")
